#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-03-18 23:40
# Author : fyt
# File   : apiAssert.py

import requests, jsonpath, operator, json

from ytApiTest import apiData, apiRequest


class AssertException(AssertionError):
	
	def __init__(self, errorInfo):
		self.errorInfo = str(errorInfo)
	
	def __str__(self):
		return self.errorInfo


class InterFaceAssert():
	
	def __init__(self):
		
		self.parsing_data = apiData.ParsingData()
		self.request = apiRequest.InterFaceReq()
	
	def error_info(self, response_data: requests.Response):
		
		return '未从返回值中查找到对比数据\n\n' \
		       'URL = {URL}\n\n' \
		       'param = {param}\n\n' \
		       'response = {response}'.format(URL=response_data.url,
		                                      param=response_data.request.body,
		                                      response=response_data.text)
	
	def find_interface_assert_value(self, response_data: requests.Response, json_expr: str):
		'''
		根据json_path表达式查找接口发返回值对应对比数据
		:param response_data: 接口返回response对象
		:param expr: json_path表达式
		:return:
		'''
		
		if operator.eq(json_expr, None):
			return self.parsing_data.parse_response_data(response_data=response_data)
		
		response_json = self.parsing_data.parse_response_data(response_data)
		assert response_json, self.request.send_case_error_info(
			'无法解析返回值{response_data}'.format(response_data=response_data))
		find_value = jsonpath.jsonpath(response_json, json_expr)
		assert find_value, self.request.send_case_error_info(error_info=self.error_info(response_data))
		
		return find_value[0]
	
	def assert_include(self, response_data, assert_value, json_expr, **kwargs):
		'''
		判断是否包含
		:param response_data: 接口返回数据
		:param assert_value: 断言数据
		:param json_expr: jsonpath路径
		:return:
		'''
		find_value = self.find_interface_assert_value(response_data=response_data,
		                                              json_expr=json_expr)
		default_bool = False
		des = self.parsing_data.get_interface_des(interface_name=kwargs.get('interface_name'),
		                                          assert_name=kwargs.get('assert_name'))
		
		try:
			if isinstance(assert_value, dict) and isinstance(find_value, dict):
				error_info = '\ndes:{des}' \
				             '\n\nresponse: {response}' \
				             '\n\n assert: {assert}' \
				             '\n\n url: {url}' \
				             '\n\n params: {params}' \
				             '\n\nheaders: {headers}'
				
				for key, value in assert_value.items():
					if find_value.__contains__(key):
						default_bool = True
					assert operator.eq(find_value[key], value), self.request.send_case_error_info(
						error_info.format_map
							(
							{'des': des,
							 'response': {key: find_value[key]},
							 'assert': {key: value},
							 'url': response_data.url,
							 'params': response_data.request.body,
							 'headers': response_data.request.headers
								
							 }
						)
					)
				assert default_bool, self.request.send_case_error_info(
					error_info='find_value = {find_value}'
					           '\n\nassert_value = {assert_value}'.format(
						assert_value=assert_value, find_value=find_value))
			
			elif isinstance(assert_value, list) and isinstance(find_value, list):
				for index, value in enumerate(assert_value):
					error_info = '\ndes:{des}' \
					             '\n\nresponse: {response} ' \
					             '\n\n assert: {assert}' \
					             '\n\n url: {url}' \
					             '\n\n params: {params}' \
					             '\n\nheaders: {headers}'.format_map(
						{'des': des,
						 'response': find_value,
						 'assert': assert_value,
						 'url': response_data.url,
						 'params': response_data.request.body,
						 'headers': response_data.request.headers
						 }
					)
					assert operator.ne(find_value.count(value), 0), self.request.send_case_error_info(
						error_info)
		finally:
			
			self.run_case_request(
				self.parsing_data.get_interface_tear_down_list(interface_name=kwargs.get('interface_name'),
				                                               assert_name=kwargs.get('assert_name')))
	
	def assert_eq(self, response_data, assert_value, json_expr, **kwargs):
		'''
		断言
		:param response_data: 接口返回值
		:param assert_value: 请求数据
		:param json_expr: jsonpath表达式
		:return:
		'''
		find_value = self.find_interface_assert_value(response_data=response_data,
		                                              json_expr=json_expr)
		des = self.parsing_data.get_interface_des(interface_name=kwargs.get('interface_name'),
		                                          assert_name=kwargs.get('assert_name'))
		error_info = '\ndes{des}\n\n断言值为空{assert_value}'
		assert operator.ne(assert_value, None), self.request.send_case_error_info(
			error_info.format(assert_value=assert_value,
			                  des=des))
		
		try:
			if isinstance(find_value, dict) and isinstance(assert_value, dict):
				print(find_value, '\n\n', assert_value)
				self.assert_dict_eq(response_dic=find_value,
				                    assert_dic=assert_value,
				                    response_data=response_data,
				                    interface_name=kwargs.get('interface_name'),
				                    assert_name=kwargs.get('assert_name'))
			
			elif isinstance(find_value, list) and isinstance(assert_value, list):
				self.assert_list_eq(response_list=find_value,
				                    assert_list=assert_value,
				                    response_data=response_data,
				                    interface_name=kwargs.get('interface_name'),
				                    assert_name=kwargs.get('assert_name'))
		
		finally:
			self.run_case_request(
				self.parsing_data.get_interface_tear_down_list(interface_name=kwargs.get('interface_name'),
				                                               assert_name=kwargs.get('assert_name')))
	
	def assert_length_eq(self, response_value, assert_value, **kwargs):
		'''
		判断对比数据长度
		:param response_value:
		:param assert_value:
		:return:
		'''
		des = self.parsing_data.get_interface_des(interface_name=kwargs.get('interface_name'),
		                                          assert_name=kwargs.get('assert_name'))
		error_info = '\ndes:{des}\n\nresponse_length: {response_length} \n\n assert_length: {assert_length}'.format_map(
			{'response_length': len(response_value),
			 'assert_length': len(assert_value),
			 'des': des})
		
		assert operator.eq(len(response_value), len(assert_value)), self.request.send_case_error_info(
			error_info)
	
	def assert_dict_eq(self, response_dic: dict, assert_dic: dict, **kwargs):
		'''
		判断字典是否相等
		:param response_dic: 返回值
		:param assert_dic: 断言字典
		:return:
		'''
		self.assert_length_eq(response_value=response_dic,
		                      assert_value=assert_dic)
		error_info = '\ndes={des}' \
		             '\n\nurl= {url}' \
		             '\n\nparam= {param}' \
		             '\n\nresponse={response} ' \
		             '\n\n assert={assert}'
		des = self.parsing_data.get_interface_des(interface_name=kwargs.get('interface_name'),
		                                          assert_name=kwargs.get('assert_name'))
		for key, value in assert_dic.items():
			assert operator.eq(response_dic[key], value), self.request.send_case_error_info(
				error_info.format_map(
					{'response': {key: response_dic[key]},
					 'assert': {key: value},
					 'des': des,
					 'url': kwargs.get('response_data').url,
					 'param': kwargs.get('response_data').request.body}))
	
	def assert_list_eq(self, response_list: list, assert_list: list, **kwargs):
		'''
		判断列表数据是否相等
		:param response_list:
		:param assert_list:
		:return:
		'''
		self.assert_length_eq(response_value=response_list,
		                      assert_value=assert_list)
		
		des = self.parsing_data.get_interface_des(interface_name=kwargs.get('interface_name'),
		                                          assert_name=kwargs.get('assert_name'))
		
		error_info = '\ndes={des}' \
		             '\n\nurl= {url}' \
		             '\n\nparam= {param}' \
		             '\n\nresponse={response} ' \
		             '\n\n assert={assert}'
		#:列表要排序在对比
		assert_list.sort()
		response_list.sort()
		for index, value in enumerate(assert_list):
			assert operator.eq(response_list[index], value), self.request.send_case_error_info(
				error_info.format_map(
					{'response': response_list[index],
					 'assert': value,
					 'des': des,
					 'url': kwargs.get('response_data').url,
					 'param': kwargs.get('response_data').request.body
					 }))
	
	def assert_response_url_status(self, response):
		'''
		断言返回值中所有URL是否可以正常访问
		:param response: 后台返回值
		:return:
		'''
		
		response_str = json.dumps(self.parsing_data.parse_response_data(response))
		
		for rep_value in response_str.split(','):
			if rep_value.rfind('https') != -1:
				url = str(rep_value[rep_value.rfind('https'):]).replace("\"", '').replace(',', '')
				requests.packages.urllib3.disable_warnings()
				body = requests.get(self.rem_special_chars(url), verify=False)
				error_info = {url: body.status_code}
				assert operator.eq(body.status_code, 200), self.request.send_case_error_info(
					'\n状态码错误{error_info}'.format(error_info=error_info))
	
	def rem_special_chars(self, string: str):
		'''
		删除特殊大括号中括号空格特殊字符
		:param string:
		:return:
		'''
		
		remap = {
			ord("{"): None,
			ord("["): None,
			ord("}"): None,
			ord(']'): None,
			ord(' '): None,
			ord('\"'): None,
			ord("\'"): None
			
		}
		
		return string.translate(remap)
	
	def assert_url_status_code(self, response_data, **kwargs):
		'''
		断言url状态是否200
		:param response_data:
		:param kwargs:
		:return:
		'''
		des = self.parsing_data.get_interface_des(interface_name=kwargs.get('interface_name'),
		                                          assert_name=kwargs.get('assert_name'))
		error_info = '\ndes: {des}' \
		             '\n\n{info}'.format_map({'des': des,
		                                      'info': {response_data.url: response_data.status_code}})
		assert operator.eq(response_data.status_code, 200), self.request.send_case_error_info(error_info=error_info)
	
	def run_case_request(self, request_list: list):
		'''
		用例前置和后置操作
		:param request_list:
		:return:
		'''
		if request_list != None and len(request_list) != 0:
			
			for dic in request_list:
				response_data = self.request.post(interface_name=dic['interface_name'],
				                                  assert_name=dic['assert_name'],
				                                  host_key=dic.get('host_key'))
				
				self.assert_url_status_code(response_data=response_data,
				                            interface_name=dic['interface_name'],
				                            assert_name=dic['assert_name'])


if __name__ == '__main__':
	pass



#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-03-18 23:39
# Author : fyt
# File   : apiRequest.py

import requests,json


from ytApiTest.apiData import ParsingData
from ytApiTest.apiAssert import InterFaceAssert
from dingtalkchatbot.chatbot import DingtalkChatbot


class InterFaceReq():
	
	def __init__(self):
		
		self.parsing_data = ParsingData()
	
	def get_interface_cookie(self, url: str, host_key=None):
		'''
		获取并保存接口cookis
		:param url: 完整接口url
		:return:
		'''
		
		
		if host_key != None:
			
			cookie_key = host_key
		else:
			
			cookie_key = self.parsing_data.get_interface_url_host_key(url=url)
		
		response_data = self.parsing_data.get_interface_response_data()
		if response_data.__contains__(cookie_key):
			return response_data[cookie_key]
		
		interface_name = self.parsing_data.get_interface_url_interface_name(host_key=cookie_key)
		login_url = self.parsing_data.get_interface_url(interface_name=interface_name, host_key=cookie_key)
		requests_data = self.parsing_data.get_interface_request_data(interface_name=interface_name,
		                                                             assert_name=cookie_key)
		response = requests.post(url=login_url,
		                         data=requests_data)
		
		if response.status_code == 200:
			
			if response.request._cookies:
				headers = response.request.headers
			
			elif response.headers['Content-Type'] != 'text/html; charset=UTF-8':
				
				if response.json().__contains__('data') and response.json()['data'].__contains__('userinfo'):
				
					headers = {'User-Agent': 'python-requests/2.22.0',
					           'Accept-Encoding': 'gzip, deflate',
					           'Accept': '*/*',
					           'Connection': 'keep-alive',
					           'Cookie': 'userId={userId}; sessionId={sessionId}; SMclient=MicroMessenger; SMmodel=Xiaomi_MI_8; SMsystem=Android_8.1.0; SMver=7.0.3; SMdisplay=393x818; SDKVersion=2.6.1; weId=supermonkey-weapp-gear; version=1.2.0;'.format(
						           userId=response.json()['data']['userinfo']['userId'],
						           sessionId=response.json()['data']['sessionId']),
					           'Content-Length': '0'
					           }
				else:
					
					headers = {'User-Agent': 'python-requests/2.23.0', 'Accept-Encoding': 'gzip, deflate', 'Accept': '*/*',
			        'Connection': 'keep-alive', 'Content-Length': '11',
			        'Content-Type': 'application/json'}
			
			self.parsing_data.save_response_data(response={cookie_key: headers})
			
			return headers
	
	def get(self, interface_name, assert_name, host_key=None):
		'''
		get 请求
		:param interface_name: 接口名称
		:param assert_name: 接口断言名称
		:param host_key: 拼接URL host名称
		:return:
		'''
		url = self.parsing_data.get_interface_url(interface_name=interface_name, host_key=host_key)
		params = self.parsing_data.get_interface_request_data(interface_name=interface_name, assert_name=assert_name)
		
		headers = self.get_interface_cookie(url=url, host_key=host_key)
		
		response = requests.get(url=url,
		                        params=params,
		                        headers=headers)
		
		InterFaceAssert().assert_url_status_code(response_data=response,
		                                    interface_name=interface_name,
		                                    assert_name=assert_name)
		self.parsing_data.save_response_data(response)
		
		return response
	
	def post(self, interface_name, assert_name, host_key=None):
		'''
		post 请求
		:param interface_name: 接口名称
		:param assert_name: 接口断言名称
		:param host_key: 拼接URL host名称
		:return:
		'''
		
		url = self.parsing_data.get_interface_url(interface_name=interface_name, host_key=host_key)
		params = self.parsing_data.get_interface_request_data(interface_name=interface_name, assert_name=assert_name)
		headers = self.get_interface_cookie(url=url, host_key=host_key)
		requests.packages.urllib3.disable_warnings()
		if headers.get('Content-Type') == 'application/json;':
			params = json.dumps(params)
		response = requests.post(url=url,
		                         data=params,
		                         headers=headers,
		                         verify=False)
		
		InterFaceAssert().assert_url_status_code(response_data=response,
		                                    interface_name=interface_name,
		                                    assert_name=assert_name)
		self.parsing_data.save_response_data(response)
		return response
	
	def send_case_error_info(self, error_info):
		'''
		发送错误消息到钉钉群
		:param error_info:
		:return:
		'''
		DingtalkChatbot(self.parsing_data.get_send_error_info_url()).send_text(error_info)
		
		return error_info


if __name__ == '__main__':
	pass



#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-03-18 23:40
# Author : fyt
# File   : apiData.py

import ytApiTest
import os, yaml, operator, jsonpath, requests,json,re
from urllib.parse import urlparse


class YAML_CONFIG_KEY():
	OBJECT_HOST = 'OBJECT_HOST'
	INTERFACE_URL = 'url'
	INTERFACE_REQUEST_DATA = 'req_data'
	INTERFACE_ASSERT_DATA = 'ast_data'
	INTERFACE_CASE_DES = 'des'
	DING_TALK_URL = 'DING_TALK_URL'
	INTERFACE_JSON_PATH = 'json_expr'
	INTERFACE_ASSERT_DATA_SETUP = 'setup'
	INTERFACE_REQUEST_DATA_TEARDOWN = 'tearDown'
	


class FindFile():
	
	def get_yaml_path(self):
		'''
        查找数据文件
        :return:
        '''
		for dirpath, dirnames, filenames in os.walk('./'):
			
			if len(filenames):
				
				for index, file_name in enumerate(filenames):
					
					if bool(os.path.splitext(file_name).count('.yaml')):
						return os.path.join(dirpath, file_name)


class YamlSingleton():
	_obj = None
	_init_flag = True
	yaml_data = None
	res_data = dict()
	
	def __new__(cls, *args, **kwargs):
		
		if YamlSingleton._obj == None:
			YamlSingleton._obj = object.__new__(cls)
		
		return cls._obj
	
	def __init__(self):
		
		if YamlSingleton._init_flag:
			YamlSingleton._init_flag = False
			YamlSingleton.yaml_data = self.get_yaml_data()
			YamlSingleton.res_data = self.res_data
	
	def get_yaml_data(self):
		'''
        获取yaml测试数据
        :return:
        '''
		
		yaml_file_path = FindFile().get_yaml_path()
		
		assert yaml_file_path, AssertionError('未找到yaml数据文件')
		
		with open(yaml_file_path, encoding='UTF-8') as f:
			dic = yaml.load(f, Loader=yaml.FullLoader)
			return dic
	
	def update_response_data(self, response: dict):
		'''
        更新接口返回数据
        '''
		self.res_data.update(response)


class ParsingData():
	
	def __init__(self):
		
		self.yaml_data = YamlSingleton().yaml_data
		self.response_data =YamlSingleton().res_data
		self.yaml_key = YAML_CONFIG_KEY
		
	
	def get_interface_data(self, interface_name, assert_name, yaml_config_key):
		
		'''
        获取接口数据
        :param interface_name: 接口名称
        :param assert_name: 接口对应断言名称
        :param yaml_config_key: yaml配置key
        :return:
        '''
		
		if self.yaml_data.__contains__(interface_name) and \
				self.yaml_data[interface_name].__contains__(assert_name):
			
			if self.yaml_data[interface_name][assert_name].__contains__(yaml_config_key):
				return self.yaml_data[interface_name][assert_name][yaml_config_key]
	
	def get_object_host(self, host_key: str = None):
		'''
        获取项目host ，默认返回第一个HOST
        :param host_key:
        :return:
        '''
		if operator.eq(host_key, None):
			
			return iter(self.yaml_data[YAML_CONFIG_KEY.OBJECT_HOST].values()).__next__()
		
		else:
			
			if self.yaml_data[YAML_CONFIG_KEY.OBJECT_HOST].__contains__(host_key):
				return self.yaml_data[YAML_CONFIG_KEY.OBJECT_HOST][host_key]
	
	def get_interface_url(self, interface_name: str, host_key: str = None):
		'''
        获取接口URL路径
        :param interface_name: 接口名称
        :param host_key: 项目host_key
        :return:
        '''
		if self.yaml_data.__contains__(interface_name):
			
			url = self.yaml_data[interface_name][YAML_CONFIG_KEY.INTERFACE_URL]
			
			if url.find('http') != -1:
				
				return url
			
			else:
				
				return self.get_object_host(host_key=host_key) + url
	
	def get_interface_request_data(self, interface_name, assert_name):
		'''
        获取接口请求数据
        :param interface_name: 接口名称
        :param assert_name: 断言名称
        :return:
        '''
		
		request_data = self.get_interface_data(interface_name=interface_name,
		                                       assert_name=assert_name,
		                                       yaml_config_key=YAML_CONFIG_KEY.INTERFACE_REQUEST_DATA)
		
		if request_data != None:
			return self.find_interface_request_data_json_path(request_data=request_data)
		
		return request_data
	
	def get_interface_assert_value(self, interface_name, assert_name):
		
		'''
        获取接口断言数据
        :param interface_name: 接口名称
        :param assert_name:  接口对应断言名称
        :return:
        '''
		
		assert_value = self.get_interface_data(interface_name=interface_name,
		                               assert_name=assert_name,
		                               yaml_config_key=YAML_CONFIG_KEY.INTERFACE_ASSERT_DATA)
		
		
		return self.replace_assert_value_json_path(assert_value)
	
	def get_interface_setup_list(self,interface_name,assert_name):
		'''
		获取前置操作接口数据
		:param interface_name: 接口名称
		:param assert_name: 接口关联断言名称
		:return:
		'''
		
		return self.get_interface_data(interface_name=interface_name,
		                               assert_name=assert_name,
		                               yaml_config_key=self.yaml_key.INTERFACE_ASSERT_DATA_SETUP)
	
	def get_interface_tear_down_list(self,interface_name,assert_name):
		'''
		获取用例后置操作
		:param interface_name: 接口名称
		:param assert_name: 接口关联断言名称
		:return:
		'''
		return self.get_interface_data(interface_name=interface_name,
		                        assert_name=assert_name,
		                        yaml_config_key=self.yaml_key.INTERFACE_REQUEST_DATA_TEARDOWN)
	
	
	def get_interface_des(self, interface_name, assert_name):
		
		'''
        获取用例说明
        :param interface_name: 接口名称
        :param assert_name: 接口对应断言名称
        :return:
        '''
		
		return self.get_interface_data(interface_name=interface_name,
		                               assert_name=assert_name,
		                               yaml_config_key=YAML_CONFIG_KEY.INTERFACE_CASE_DES)
	
	def get_interface_json_path(self, interface_name, assert_name):
		
		'''
        获取用例jsonpath
        :param interface_name: 接口名称
        :param assert_name: 接口对应断言名
        :return:
        '''
		
		return self.get_interface_data(interface_name=interface_name,
		                               assert_name=assert_name,
		                               yaml_config_key=YAML_CONFIG_KEY.INTERFACE_JSON_PATH)
	
	def get_interface_url_host_key(self, url: str):
		
		'''
		获取URL对应HOST key值
		:param url: url
		:return:
		'''
		
		object_host_dict = self.yaml_data[YAML_CONFIG_KEY.OBJECT_HOST]
		url_netloc = urlparse(url).netloc
		for key, value in object_host_dict.items():
			
			if operator.eq(urlparse(value).netloc, url_netloc):
				return key
	
	def get_interface_url_interface_name(self, host_key: str):
		'''
		通过hostkey获取接口名称
		:param host_key:
		:return:
		'''
		
		for interface_name, value in self.yaml_data.items():
			if value.__contains__(host_key) and operator.ne(interface_name, YAML_CONFIG_KEY.OBJECT_HOST):
				return interface_name
	
	def get_interface_response_data(self):
		'''
		获取接口返回值
		:return:
		'''
		return self.response_data
	
	def get_send_error_info_url(self):
		'''
        获取项目配置钉钉机器人URL
        :return:
        '''
		return self.yaml_data[YAML_CONFIG_KEY.DING_TALK_URL]
	
	def get_interface_assert_name(self, assert_value: dict):
		'''
        获取接口断言key
        :param assert_value: 断言值
        :return:
        '''
	
	def update_interface_json_path(self, interface_name, assert_name, new_value: dict):
		'''
        修改json_path 路径
        :param interface_name: 接口名称
        :param assert_name: 断言名称
        :param new_value: 修改值，以字典传入
        :return:
        '''
		old_json_path = self.get_interface_json_path(interface_name=interface_name,
		                                             assert_name=assert_name)
		
		return old_json_path.format(**new_value)
	
	def update_interface_request_data(self, interface_name, assert_name, new_request_data: dict):
		'''
        修改接口请求参数
        :param interface_name: 接口名称
        :param assert_name: 断言名称
        :param new_request_data: 新接口请求值
        '''
		
		old_interface_request_data = self.get_interface_request_data(interface_name=interface_name,
		                                                             assert_name=assert_name)
		
		if old_interface_request_data != None:
			old_interface_request_data.update(new_request_data)
	
	def find_interface_request_data_json_path(self, request_data: dict):
		'''
        替换请求参数里通过jonspath查找到的值
        :param request_data: 请求字典
        :return:
        '''
		for key, value in request_data.items():
			
			if isinstance(value, str) and value.find('$') != -1:
				
				find_value = jsonpath.jsonpath(self.response_data, value)
				
				if find_value:
					request_data[key] = find_value[0]
		
		return request_data
	
	def save_response_data(self, response: requests.Response):
		'''
        保存接口返回值
        :param dic:
        :return:
        '''
		
		if isinstance(response, dict):
			
			json_value = response
		
		elif response.status_code == 200:
			json_key = os.path.split(urlparse(response.request.url).path)[-1]
			json_key = json_key.replace('.', '/')
			json_value = {json_key: self.parse_response_data(response_data=response)}
		
		else:
			
			raise ValueError('接口返回值解析错误{}'.format({response.url: response.status_code}))
			return
		
		YamlSingleton().update_response_data(response=json_value)
	
	def parse_response_data(self, response_data: requests.Response):
		'''
        解析接口返回对象为json
        :param response_data:
        :return:
        '''
		
		if isinstance(response_data, requests.Response) and operator.eq(response_data.headers['Content-Type'],
		                                                                'application/json; charset=UTF-8'):
			return response_data.json()
		
		elif isinstance(response_data, dict):
			return response_data
	
	def replace_assert_value_json_path(self,assert_value):
		'''
		替换值
		:param assert_value:
		:return:
		'''
		temp_str = json.dumps(assert_value,ensure_ascii=False)
		
		if operator.eq(temp_str,None) or temp_str.find('$') == -1:
			return json.loads(temp_str)
		json_path_list = re.compile(r'(\$.*?,)',re.I).findall(temp_str)
		
		for index,value in enumerate(json_path_list):
			json_expr = self.delete_special_character(value)
			s = temp_str.replace(json_expr,self.find_json_expr_value(value))
			temp_str = s
		
		return json.loads(temp_str)

	def delete_special_character(self,character:str):
		
		'''
		删除特殊字符
		:param character:
		:return:
		'''
		
		remap = {
			ord("\""): None,
			ord(","): None,
			ord("\\"): None,
		}
		
		return character.translate(remap)
		
		
	def find_json_expr_value(self,json_expr):
		'''
		查找json_expr 返回值
		:param json_expr:
		:return:
		'''
		
		if jsonpath.jsonpath(self.response_data,json_expr):
			
			return jsonpath.jsonpath(self.response_data,json_expr)
		
		elif jsonpath.jsonpath(self.yaml_data,json_expr):
			
			return jsonpath.jsonpath(self.yaml_data,json_expr)
		
		else:
			return json_expr
		
		


if __name__ == '__main__':
	pass
	
	
	



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-03-20 15:27
# @Author  : SuperMonkey
# @Site    : 
# @File    : api.py
# @Software: PyCharm

from ytApiTest.apiRequest import InterFaceReq
from ytApiTest.apiAssert import InterFaceAssert
from ytApiTest.apiData import ParsingData

def get(interface_name, assert_name, host_key=None):
	return InterFaceReq().get(interface_name=interface_name,
	                          assert_name=assert_name,
	                          host_key=host_key)


def post(interface_name, assert_name, host_key=None):
	return InterFaceReq().post(interface_name=interface_name,
	                           assert_name=assert_name,
	                           host_key=host_key)


def get_interface_url(interface_name: str, host_key: str = None):
	'''
	获取接口URL路径
	:param interface_name: 接口名称
	:param host_key: 项目host_key
	:return:
	'''
	return ParsingData().get_interface_url(interface_name=interface_name,
	                                       host_key=host_key)


def get_interface_request_data(interface_name, assert_name):
	'''
	获取接口请求数据
	:param interface_name: 接口名称
	:param assert_name: 断言名称
	:return:
	'''
	return ParsingData().get_interface_request_data(interface_name=interface_name,
	                                                assert_name=assert_name)


def get_interface_case_assert_data(interface_name, assert_name):
	'''
	获取接口断言数据
	:param interface_name: 接口名称
	:param assert_name:  接口对应断言名称
	:return:
	'''
	return ParsingData().get_interface_assert_value(interface_name=interface_name,
	                                                assert_name=assert_name)


def get_interface_json_path(interface_name, assert_name):
	'''
	获取用例jsonpath
	:param interface_name: 接口名称
	:param assert_name: 接口对应断言名
	:return:
	'''
	
	return ParsingData().get_interface_json_path(interface_name=interface_name,
	                                             assert_name=assert_name)
def get_interface_des(interface_name, assert_name):
	
	return ParsingData().get_interface_des(interface_name=interface_name,
	                                       assert_name=assert_name)

def update_interface_json_path(interface_name, assert_name, new_value: dict):
	'''
	修改json_path 路径
	:param interface_name: 接口名称
	:param assert_name: 断言名称
	:param new_value: 修改值，以字典传入
	:return:
	'''
	return ParsingData().update_interface_json_path(interface_name=interface_name,
	                                                assert_name=assert_name,
	                                                new_value=new_value)


def update_interface_request_data(interface_name, assert_name, new_request_data: dict):
	'''
	修改接口请求参数
	:param interface_name: 接口名称
	:param assert_name: 断言名称
	:param new_request_data: 新接口请求值
	'''
	
	return ParsingData().update_interface_request_data(interface_name=interface_name,
	                                                   assert_name=assert_name,
	                                                   new_request_data=new_request_data)


def assert_body_eq_assert_value(response_data=None, assert_value=None, json_expr=None,**kwargs):
	'''
	断言
	:param response_data: 接口返回值
	:param assert_value: 请求数据
	:param json_expr: jsonpath表达式
	'''
	
	if kwargs.__contains__('interface_name') and kwargs.__contains__('assert_name'):
		InterFaceAssert().run_case_request(
			request_list=ParsingData().get_interface_setup_list(interface_name=kwargs.get('interface_name'),
			                                                    assert_name=kwargs.get('assert_name')))
		response_data = post(interface_name=kwargs.get('interface_name'),assert_name=kwargs.get('assert_name'),host_key=kwargs.get('host_key'))
		assert_value = get_interface_case_assert_data(interface_name=kwargs.get('interface_name'),assert_name=kwargs.get('assert_name'))
		json_expr = get_interface_json_path(interface_name=kwargs.get('interface_name'),assert_name=kwargs.get('assert_name'))
	
	InterFaceAssert().assert_eq(response_data=response_data,
	                                              assert_value=assert_value,
	                                              json_expr=json_expr,**kwargs)


def assert_body_include_value(response_data=None,assert_value=None,json_expr=None,**kwargs):
	'''
	判断是否包含
	:param response_data: 接口返回数据
	:param assert_value: 断言数据
	:param json_expr: jsonpath路径
	:return:
	'''
	
	if kwargs.__contains__('interface_name') and kwargs.__contains__('assert_name'):
		
		InterFaceAssert().run_case_request(request_list=ParsingData().get_interface_setup_list(interface_name=kwargs.get('interface_name'),
		                                                                                       assert_name=kwargs.get('assert_name')))
		
		response_data = post(interface_name=kwargs.get('interface_name'), assert_name=kwargs.get('assert_name'),
		                     host_key=kwargs.get('host_key'))
		assert_value = get_interface_case_assert_data(interface_name=kwargs.get('interface_name'),
		                                              assert_name=kwargs.get('assert_name'))
		json_expr = get_interface_json_path(interface_name=kwargs.get('interface_name'),
		                                    assert_name=kwargs.get('assert_name'))
	
	InterFaceAssert().assert_include(response_data=response_data,
	                                            assert_value=assert_value,
	                                            json_expr=json_expr,
	                                            **kwargs
	                                            )


def assert_response_url_status(response,**kwargs):
	'''
	断言返回值中所有URL是否可以正常访问
	:param response: 后台返回值
	:return:
	'''
	if kwargs.__contains__('interface_name') and kwargs.__contains__('assert_name'):
		
		response = post(interface_name=kwargs.get('interface_name'), assert_name=kwargs.get('assert_name'),
		                     host_key=kwargs.get('host_key'))
	
	InterFaceAssert().assert_response_url_status(response=response)


	

if __name__ == '__main__':
 
	pass


#!/usr/local/bin/python3
# -*- coding:utf-8 -*-
# Time   : 2020-01-07 19:57
# Author : fyt
# File   : setup.py


from setuptools import setup


def readme_data():
    with open('./README.rst', encoding='utf-8') as f:
        data = f.read()

        return data


setup(
    name='ytApiTest',
    version='1.0.32',
    author='fyt',
    author_email='fangyt@163.com',
    packages=["ytApiTest"],
    install_requires=[
        'PyYAML',
        'requests',
        'jsonpath',
        'DingtalkChatbot',
        'pytest',
        'pytest-parallel'
    ],
    long_description=readme_data(),
    url='https://github.com/fangyt/ytApiTest.git'
)
